package compiler

import (
	"git.urbach.dev/cli/q/src/asm"
	"git.urbach.dev/cli/q/src/core"
)

// eachFunction recursively finds all the calls to external functions.
// It avoids calling the same function twice with the help of a hashmap.
func (r *Result) eachFunction(caller *core.Function, traversed map[*core.Function]bool, call func(*core.Function)) {
	call(caller)
	traversed[caller] = true

	for _, x := range caller.Assembler.Instructions {
		if x.Mnemonic != asm.CALL {
			continue
		}

		label, isLabel := x.Data.(*asm.Label)

		if !isLabel {
			continue
		}

		callee, exists := r.Functions[label.Name]

		if !exists {
			continue
		}

		if traversed[callee] {
			continue
		}

		r.eachFunction(callee, traversed, call)
	}
}
